/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include "data.h"
#include "router.h"
#include "network.h"
#include "machine.h"
#include "interface.h"
#include "subnet.h"
#include "misc.h"

Data::Data()
{
}

Data::~Data()
{
	Network *n;
	Machine *m;
	Subnet *s;
	Router *r;
	Link *ln;

	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{ s = (Subnet*) ln->item; delete s; }
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{ m = (Machine*) ln->item; delete m; }
	for (ln=routers.get_head(); ln!=NULL; ln=ln->next)
	{ r = (Router*) ln->item; delete r; }
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{ n = (Network*) ln->item; delete n; }
}

void Data::save(FILE *f)
{
	int i, j, k;
	Network *n;
	Subnet *s;
	Router *r;
	Link *ln;

	write_int(f, subnets.size());
	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (Subnet*) ln->item;
		s->save(f);
	}

	write_int(f, routers.size());
	for (ln=routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		r->save(f);
	}

	// save only those networks that are attached to stuff
	k = 0;
	j = networks.size();
	for (i=0; i<j; i++)
	{
		n = (Network*) networks.get_item(i);
		if (n->interfaces.size() + n->machines.size() > 1) k++;
	}
	write_int(f, k);
	for (i=0; i<j; i++)
	{
		n = (Network*) networks.get_item(i);
		if (n->interfaces.size() + n->machines.size() > 1)
			n->save(f);
	}
}

void Data::load(FILE *f)
{
	int i, j;
	Network *n;
	Router *r;
	Subnet *s;

	j = read_int(f);
	subnets.clear();
	for (i=0; i<j; i++)
	{
		s = new Subnet();
		s->load(f);
		subnets.tag(s);
	}

	j = read_int(f);
	routers.clear();
	for (i=0; i<j; i++)
	{
		r = new Router();
		r->load(f);
		routers.tag(r);
	}

	j = read_int(f);
	networks.clear();
	for (i=0; i<j; i++)
	{
		n = new Network();
		n->load(f, &routers, &machines, &subnets);
		networks.tag(n);
	}
}

void Data::print()
{
	printf("SUBNETS %d\n", subnets.size());
	printf("ROUTERS %d\n", routers.size());
	printf("MACHINES %d\n", machines.size());
	printf("NETWORKS %d\n", networks.size());
}

