/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <stdlib.h>
#include "network.h"
#include "router.h"
#include "subnet.h"
#include "interface.h"
#include "machine.h"
#include "misc.h"

Network::Network()
{
	clear();
}

Network::~Network()
{
}

void Network::clear()
{
	view = 0;
	x = rand()%400;
	y = rand()%300;
	w = h = 8;
	selected = 0;
	show = 1;
	type = 'n';
}

void Network::print()
{
	Subnet *sn;
	int j, k;

	j = subnets.size();
	printf("%d %d\n", interfaces.size(), j);
	for (k=0; k<j; k++)
	{
		sn = (Subnet*) subnets.get_item(k);
		sn->print();
	}
}

void Network::save(FILE *f)
{
	int i, j;
	Interface *in;
	Subnet *sn;
	Machine *m;
	Router *r;
	Link *ln;

	write_int(f, interfaces.size());
	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = (Router*) in->router;
		for (i=0; i<64; i++) fputc(r->name[i], f);
		write_int(f, in->index);
	}

	write_int(f, machines.size());
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->save(f);
	}

	j = subnets.size();
	write_int(f, subnets.size());
	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		sn = (Subnet*) ln->item;
		sn->save(f);
	}
}

void Network::load(FILE *f, List *rtr, List *mach, List *sns)
{
	int i, j, k, l, o;
	Interface *in;
	int index;
	Machine *m;
	Router *r;
	Link *ln, *ln2;
	Subnet sn, *sn2;
	char tname[65];

	j = read_int(f);
	interfaces.clear();
	for (i=0; i<j; i++)
	{
		for (k=0; k<64; k++) tname[k] = fgetc(f); tname[64] = '\0';
		index = read_int(f);
		for (ln=rtr->get_head(); ln!=NULL; ln=ln->next)
		{
			r = (Router*) ln->item;
			if (strcmp(r->name, tname)==0)
			{
				for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in = (Interface*) ln2->item;
					if (in->index == index)
					{
						in->network = this;
						interfaces.tag(in);
					}
				}
			}
		}
	}

	j = read_int(f);
	machines.clear();
	for (i=0; i<j; i++)
	{
		m = new Machine();
		m->load(f);
		m->network = this;
		machines.tag(m);
		mach->tag(m);
	}

	j = read_int(f);
	subnets.clear();
	for (i=0; i<j; i++)
	{
		sn.clear();
		sn.load(f);
		for (ln=sns->get_head(); ln!=NULL; ln=ln->next)
		{
			sn2 = (Subnet*) ln->item;
			if ((sn2->ip.compare(sn.ip)) && (sn2->mask == sn.mask))
			{
				subnets.tag(sn2);
			}
		}
	}
}

